'use strict';

var exec = require('child_process').exec;
var fs = require('fs');
var path = require('path');
var _ = require('lodash');
var chalk = require('chalk');
var processArgs = require('minimist')(process.argv.slice(2));
var semver = require('semver');
var util = require('util');
var yeoman = require('yeoman-generator');
var PackageDependencies = require('./libs/packageDependencies');
var npmDependencies = require('./dependencies/npmDependencies.json');
var isWindows = process.platform === 'win32';
var vasync = require('vasync');
var packageJson = require('../package.json');

var homeDirectory = isWindows ? process.env.USERPROFILE : process.env.HOME;

var getVellum = function() {
    var callback = this.async();

    this.remote('mobify', 'vellum', 'master', function(err, remote) {
        if (err) {
            callback(err);
        }

        remote.directory('dist', 'app/global/styles');
        callback();
    }, true);
};

// Grab the semver value from a JSON string
var getGeneratorVersionFromNpmJson = function(stdout) {
    try {
        var versionInfo = JSON.parse(stdout);
        if (typeof versionInfo === 'string') {
            return versionInfo;
        } else {
            return versionInfo['dependencies']['generator-adaptivejs']['version'];
        }
    } catch (e) {
        console.log(e);
        return undefined;
    }
};

//Check to see if project with slug already exists
var dirExists = function(slug) {
    try {
        var files = fs.readdirSync(slug);
        return true;
    } catch (err) {
        return false;
    }
};

var AdaptiveGenerator = module.exports = function AdaptiveGenerator(args, options, config) {
    yeoman.generators.Base.apply(this, arguments);

    var self = this; // Bind this reference for later

    this.on('end', function() {

        // Some tests skip install
        if (!options.skipInstall) {

            var onComplete = function() {
                console.log('All finished!');

                // Let our tests know that we have truly finished all tasks
                self.emit('adaptivejs:done');
            };

            var runNpmInstall = function(arg, callback) {
                var npmOptions = {};
                var cb = function(err) {
                    if (err) {
                        var warning = 'There was an error running npm install.\n' +
                                      'Check the permissions on your .npm folder.';
                        console.log(chalk.yellow(chalk.bold(warning)));
                        console.log(err);
                        process.exit(1);
                    }
                    callback();
                };

                this.npmInstall(null, npmOptions, cb);
            }.bind(this);

            vasync.pipeline({
                funcs: [
                    runNpmInstall
                ]
            }, onComplete);
        }

    });
};

util.inherits(AdaptiveGenerator, yeoman.generators.Base);

/**
 *  Check to see if you the user has the latest version
 *  of the generator
 */
AdaptiveGenerator.prototype.checkVersion = function() {
    var self = this;
    var done = this.async();

    // Don't check versions if we're running tests
    if (process.env.IS_RUNNING_TESTS) {
        return done();
    }

    console.log('Checking for newer versions of generator-adaptivejs...');

    exec('npm list -g generator-adaptivejs --json', function(err, stdout) {
        var localError = 'There was an error getting the local version of generator-adaptivejs';
        // Checking for result here rather then error because there are situations where you still get
        // the version information, but it comes with an error as well (like when testing using
        // npm link). As long as there is a result, we're good to continue.
        if (!stdout) {
            console.log(localError);
            console.log(err);
            return done();
        }

        var localVersion = getGeneratorVersionFromNpmJson(stdout);
        if (!localVersion) {
            console.log(localError);
            return done();
        }

        exec('npm info generator-adaptivejs version --json', function(err, stdout) {
            var latestError = 'There was an error getting the latest version of generator-adaptivejs';

            // Checking for result here rather then error because there are
            // situations where you still get the version information, but it
            // comes with an error as well (like when testing using npm link).
            // As long as there is a result, we're good to continue.
            if (!stdout) {
                console.log(latestError);
                console.log(err);
                return done();
            }

            var latestVersion = getGeneratorVersionFromNpmJson(stdout);

            if (!latestVersion) {
                console.log(latestError);
                return done();
            }

            // If the local version is not less then the latest version on the
            // npm repo, return
            if (semver.gte(localVersion, latestVersion)) {
                console.log('The version of the generator you have installed is up-to-date!');
                return done();
            }

            var prompts = {
                type: 'confirm',
                name: 'upgrade',
                message: 'We have detected that there is a newer version of generator-adaptivejs available.\n' +
                         'Would you like to upgrade? (recommended)',
                default: true
            };

            self.prompt(prompts, function(props) {
                if (props.upgrade) {
                    console.log('\nPlease run the following command:\n\n' +
                                '    npm update -g generator-adaptivejs\n\n' +
                                'When complete, please generate your project again!');
                    return process.exit(0);
                }
                return done();
            });
        });
    });
};

/**
 *  Check that git is installed so we can install dependencies
 */
AdaptiveGenerator.prototype.checkGit = function checkGit() {
    var done = this.async();

    exec('git --version', function(err) {
        if (err) {
            console.log(chalk.yellow(chalk.bold('You must have git installed to create a project')));
            console.log('http://git-scm.com/book/en/Getting-Started-Installing-Git');
            process.exit(1);
        }
        done();
    });
};

/**
 *  Checks to see if the user has already accepted the mobify licence.
 *  If not, display the licence.
 */
AdaptiveGenerator.prototype.acceptLicense = function acceptLicense() {
    var callback = this.async();
    var adaptiveGlobalSettingsPath = path.join(homeDirectory, '.adaptiverc');
    this.acceptedLicense = false;

    // Check if licence file is there, and read it
    if (fs.existsSync(adaptiveGlobalSettingsPath)) {
        this.acceptedLicense = JSON.parse(this.readFileAsString(adaptiveGlobalSettingsPath)).acceptedLicense;
    }

    // If the licence is accepted, don't display it
    if (this.acceptedLicense) {
        callback();
        return true;
    }

    var prompts = [{
        type: 'confirm',
        name: 'openLicense',
        message: 'We have a license you must read and agree to. Read license?',
        default: false
    }];

    this.prompt(prompts, function(props) {
        if (!props.openLicense) {
            callback();
        }

        // Display licence
        var license = path.join(__dirname, '../LICENSE');
        var child;

        if (isWindows) {
            child = this.spawnCommand('start', ['notepad', license]);
        } else {
            child = this.spawnCommand('less', [license]);
        }

        // After the user closes the license, ask them to accept it
        child.on('exit', function() {

            var prompts = [
                {
                    type: 'confirm',
                    name: 'acceptLicense',
                    message: 'I have read, understand, and accept the terms and conditions stated in the license above.',
                    default: false
                }
            ];

            this.prompt(prompts, function(props) {
                // Exit if terms aren't accpeted
                if (!props.acceptLicense) {
                    process.exit(0);
                }

                // Save licence file so users don't see the licence every time
                fs.writeFile(adaptiveGlobalSettingsPath, '{"acceptedLicense": true}', function(err) {
                    if (err) {
                        throw err;
                    }
                });

                callback();
            });

        }.bind(this));
    }.bind(this));

};

AdaptiveGenerator.prototype.askFor = function askFor() {
    var callback = this.async();

    var mergeDefaults = function(props) {
        var defaultsName = _.isBoolean(processArgs.defaults) ? 'default' : processArgs.defaults;
        var defaultsPath = util.format('defaults/%s.json', defaultsName);
        var defaults = JSON.parse(fs.readFileSync(path.join(__dirname, defaultsPath)));

        props = _.extend(props, defaults);

        return props;
    };

    // have Yeoman greet the user.
    console.log(this.yeoman);

    var prompts = [];
    var slugErrorMsg = 'A project with that name already exists in the current directory.';

    // If no slug was provided, ask for it
    if (!processArgs.slug) {
        prompts.push({
            type: 'input',
            name: 'projectSlug',
            message: 'What is your project name? (if using Mobify Cloud, this should be your \n  project slug):',
            validate: function(answer) {
                if (!answer.length) {
                    return 'You must enter a name.';
                }

                //check to see if project name exists. If it does, we stop the generator
                if (dirExists(answer)) {
                    return slugErrorMsg;
                }
                return true;
            }
        });
    } else {
        if (dirExists(processArgs.slug)) {
            console.log(chalk.red.bold(slugErrorMsg));
            process.exit(1);
        }
    }

    // If no slug was provided, ask for it
    if (!processArgs.url) {
        prompts.push({
            type: 'input',
            name: 'siteUrl',
            message: 'What is the URL of your project?',
            default: ''
        });
    }

    if (!processArgs.defaults) {
        prompts = prompts.concat([
            {
                type: 'list',
                name: 'selectorLibrary',
                message: 'Do you want Zepto or jQuery?',
                choices: [
                    {name: 'jQuery', value: 'jQuery'},
                    {name: 'Zepto', value: 'Zepto'}
                ]
            },
            {
                type: 'confirm',
                name: 'useSass',
                message: 'Do you want to write your CSS with Sass?'
                    + chalk.gray(' Includes Mobify’s Vellum base styles\n  (https://github.com/mobify/vellum).\n'),
                default: true
            },
            {
                when: function(response) {
                    return response.useSass;
                },
                type: 'confirm',
                name: 'useSassLinting',
                message: 'Do you want to lint your Sass files (sass-lint using Mobify\'s linting rules)?\n'
                    + chalk.gray(' Find more about sass-lint at https://github.com/sasstools/sass-lint\n'),
                default: true
            },
            {
                type: 'confirm',
                name: 'useLivingStyleGuide',
                message: 'Do you want to have a Living Styleguide?',
                default: true
            },
            {
                type: 'confirm',
                name: 'useNightwatch',
                message: 'Do you want the Nightwatch testing framework?',
                default: true
            },
            {
                type: 'confirm',
                name: 'useJSLinting',
                message: 'Do you want to lint your JavaScript files (eslint using Mobify\'s linting rules)?',
                default: true
            },
            {
                when: function(response) {
                    return response.useSassLinting || response.useJSLinting;
                },
                type: 'confirm',
                name: 'useHooks',
                message: 'Do you want to lint your project on every git push?',
                default: true
            }
        ]);
    }

    this.prompt(prompts, function(props) {
        if (processArgs.defaults) {
            props = mergeDefaults(props);
        }

        this.buildTasks = {
            dev: ['adaptive-build_dev'],
            prod: ['adaptive-build_prod'],
            preview: ['adaptive-preview'],
            push: ['adaptive-push'],
            toJSON: function(o) {
                return JSON.stringify(o).replace(/"/g, '\'');
            }
        };

        this.testTasks = {
            /* eslint-disable camelcase */
            test: ['build_dev', 'adaptive-test'],
            test_ci: ['build_prod', 'adaptive-test_ci'],
            test_skip_build: ['adaptive-test'],
            test_browser: ['build_dev', 'adaptive-test_browser'],
            /* eslint-enable camelcase */
            toJSON: function(o) {
                return JSON.stringify(o).replace(/"/g, '\'');
            }
        };

        this.npmScripts = {
            scripts: {}, // default to noop
            toJSON: function(o) {
                return JSON.stringify(o).replace(/'/g, '\"'); // use double quotes
            }
        };

        this.lintingTasks = {
            tasks: [],
            toJSON: function(o) {
                return JSON.stringify(o).replace(/'/g, '\"'); // use double quotes
            },
            printLintAllTask: function() {
                return 'grunt.registerTask(\'lint-all\', \'Task for running all linters in the project\', ' + this.toJSON(this.tasks) + ');';
            }
        };

        this.npmDependencies = new PackageDependencies(npmDependencies.adaptiveDefaults);
        this.npmDevDependencies = new PackageDependencies();

        this.generatorVersion = packageJson.version;
        this.projectSlug = props.projectSlug || processArgs.slug;

        // Prevents case where --url flag is provided without a value and is
        // interpreted as a boolean (i.e. true)
        processArgs.url = processArgs.url === true ? '' : processArgs.url;
        this.siteUrl = props.siteUrl || processArgs.url;

        this.adaptiveVersion = props.adaptiveVersion;
        this.selectorLibrary = props.selectorLibrary;
        this.useLivingStyleGuide = props.useLivingStyleGuide;
        this.useSass = props.useSass;
        this.useSassLinting = props.useSassLinting;
        this.useNightwatch = props.useNightwatch;
        this.useJSLinting = props.useJSLinting;
        this.useHooks = props.useHooks;

        if (this.useSass) {
            this.npmDependencies.add(npmDependencies.sass);

            this.buildTasks.dev.unshift('adaptive-compile-css__dev');
            this.buildTasks.prod.unshift('adaptive-compile-css__prod');
            this.buildTasks.push.unshift('adaptive-compile-css__prod');

            if (this.useSassLinting) {
                this.npmDevDependencies.add(npmDependencies.sassLinting);

                this.testTasks.test.unshift('scsslint');
                this.testTasks.test_ci.unshift('scsslint');
                this.testTasks.test_skip_build.unshift('scsslint');
                this.testTasks.test_browser.unshift('scsslint');

                this.lintingTasks.tasks.push('scsslint');
            }
        }

        if (this.useLivingStyleGuide) {
            this.buildTasks.preview.unshift('build-with-styleguide');

            this.npmDevDependencies.add(npmDependencies.livingStyleGuide);
        } else {
            this.buildTasks.preview.unshift('build_dev');
        }

        if (this.useNightwatch) {
            this.npmDevDependencies.add(npmDependencies.nightwatch);
        }

        if (this.useJSLinting) {
            this.npmDevDependencies.add(npmDependencies.jsLinting);

            this.testTasks.test.unshift('lint:dev');
            this.testTasks.test_ci.unshift('lint:prod');
            this.testTasks.test_skip_build.unshift('lint:dev');
            this.testTasks.test_browser.unshift('lint:dev');

            this.lintingTasks.tasks.push('lint:prod');
        }

        if (this.useHooks) {
            this.npmDevDependencies.add(npmDependencies.hooks);
            this.npmScripts.scripts = {'postinstall': 'grunt setup-githooks'};
        }

        // Clear the build folder first before building
        this.buildTasks.dev.unshift('adaptive-clean_build');
        this.buildTasks.prod.unshift('adaptive-clean_build');
        this.buildTasks.push.unshift('adaptive-clean_build');

        callback();

    }.bind(this));
};

AdaptiveGenerator.prototype.app = function app() {

    this.mkdir(this.projectSlug);
    this.destinationRoot(this.projectSlug);
    this.mkdir('tasks');
    this.mkdir('tasks/config');
    this.mkdir('app/vendor');

    // UI script linking for the homepage template
    this.runViewUI =
        _.template(
            fs.readFileSync(path.join(__dirname, '../view/templates/') +
                '_runViewUI.dust'), _.extend(this, {
                    viewName: 'home',
                    uiFunction: 'home'
                })
        );

    this.selectorLibPath = 'vendor/' + this.selectorLibrary.toLowerCase();

    this.directory(path.join('selector-libs', this.selectorLibrary.toLowerCase()), 'app/vendor');

    if (this.useNightwatch) {
        this.directory('tasks/nightwatch', 'tasks/nightwatch');
        this.directory('tests/system', 'tests/system');
        this.template('tests/_site.json', 'tests/system/site.json');
    }

    this.directory('app', 'app');
    this.directory('static', 'static');
    this.directory('tests/integration', 'tests/integration');
    this.directory('tests/unit', 'tests/unit');
    this.directory('tests/fixtures', 'tests/fixtures');
    this.directory('tests/runner', 'tests/runner');
    this.copy('gitignore', '.gitignore');
    this.copy('editorconfig', '.editorconfig');

    // only include the circle.yml template in mobify defaults projects
    if (processArgs.defaults === 'mobify') {
        this.copy('circle.yml', 'circle.yml');
    }

    this.template('_package.json', 'package.json');
    this.template('_Gruntfile.js', 'Gruntfile.js');
    this.template('tests/_config.js', 'tests/runner/config.js');
    this.template('_home-template.dust', 'app/pages/home/home.dust');
    this.template('_README.md', 'README.md');

    if (this.useJSLinting) {
        this.directory('tasks/lint', 'tasks/lint');

        // Copy over eslint files
        this.directory('tasks/eslint', 'tasks/eslint');
        this.copy('tasks/config/eslint.js', 'tasks/config/eslint.js');

        this.copy('jslinting.js', 'tasks/jslinting.js');
    }

    if (this.useHooks) {
        // Copy the boilerplate tasks files
        this.directory('tasks/hooks', 'tasks/hooks');
        this.directory('githooks', 'githooks');
        this.directory('tasks/githooks-env', 'tasks/githooks-env');
        this.copy('tasks/config/githooks.js', 'tasks/config/githooks.js');
    }

    if (this.useSass) {
        this.directory('tasks/compile-css', 'tasks/compile-css');

        this.copy('styles/stylesheet.scss', 'app/stylesheet.scss');
        this.copy('styles/_pages.scss', 'app/global/styles/_pages.scss');
        this.copy('styles/_components.scss', 'app/global/styles/_components.scss');
        this.copy('styles/_utilities.scss', 'app/global/styles/_utilities.scss');

        this.copy('styles/_header.scss', 'app/global/includes/header/_header.scss');
        this.copy('styles/_home.scss', 'app/pages/home/_home.scss');


        this.copy('tasks/config/autoprefixer.js', 'tasks/config/autoprefixer.js');
        if (!this.useLivingStyleGuide) {
            this.copy('tasks/config/watch.js', 'tasks/config/watch.js');
            this.copy('tasks/config/sass.js', 'tasks/config/sass.js');
        }

        if (this.useSassLinting) {
            this.directory('tasks/sasslint', 'tasks/sasslint');
            this.copy('tasks/config/sasslint.js', 'tasks/config/sasslint.js');
            this.copy('scsslinting.js', 'tasks/scsslinting.js');
        }

        getVellum.call(this);
    } else {
        // this.fs.delete('app/**/*.scss'); // Requires newer version of Yeoman
        this.copy('_stylesheet.css', 'static/css/stylesheet.css');
    }

    if (this.useLivingStyleGuide) {
        console.log('Adaptivejs:Invoke:LivingStyleGuide');

        this.invoke('adaptivejs:styleguide',
            {
                options: {
                    projectSlug: this.projectSlug,
                    confirmLivingStyleGuide: true
                }
            }
        );
    }
};
