'use strict';

var util = require('util');
var path = require('path');
var yeoman = require('yeoman-generator');
var chalk = require('chalk');
var fs = require('fs');

var GithooksGenerator = yeoman.generators.Base.extend({
    prompting: function() {
        var done = this.async();
        this.prompt([
            {
                type: 'confirm',
                name: 'confirmJS',
                message: 'WARNING: The githooks sub-generator is only compatible with Adaptive.js projects that use eslint. Do you wish to proceed?',
                default: false
            },
            {
                when: function(response) {
                    return response.confirmJS;
                },
                type: 'confirm',
                name: 'includeSass',
                message: 'Do you want to also include Sass linting in the githook?',
                default: false
            },
            {
                when: function(response) {
                    return response.includeSass;
                },
                type: 'confirm',
                name: 'confirmSass',
                message: 'WARNING: The githooks sub-generator is only fully compatible with Adaptive.js projects that use sass-lint. Do you wish to proceed?',
                default: false
            }
        ], function(answers) {
            this.confirmJS = answers.confirmJS;
            if (!this.confirmJS) {
                process.exit(0);
            }
            this.includeSass = answers.includeSass;
            this.confirmSass = answers.confirmSass;
            if (this.includeSass && !this.confirmSass) {
                process.exit(0);
            } else {
                done();
            }
        }.bind(this));
    },

    init: function() {
        this.log(chalk.yellow(
            'Updating package.json with grunt-githooks dependency and setting up Mobify\'s git hooks...\n'
        ));
    }
});

GithooksGenerator.prototype.app = function app() {
    // Read existing package.json
    var packageJsonPath = './package.json';
    var packageJsonString = this.readFileAsString(packageJsonPath);
    var packageJson = JSON.parse(packageJsonString);

    // Read dependencies from generator to get grunt-githooks version
    var generatorDependencies = JSON.parse(this.readFileAsString(this.sourceRoot() + '../../../app/dependencies/npmDependencies.json'));
    var gruntGithooksVersion = generatorDependencies.hooks['grunt-githooks'];

    // Add grunt-githooks dependency to package.json
    if (packageJson.devDependencies) {
        packageJson.devDependencies['grunt-githooks'] = gruntGithooksVersion;
    } else {
        packageJson.devDependencies = {
            'grunt-githooks': gruntGithooksVersion
        };
    }

    // If grunt-githooks exists in project's dependencies, remove it
    if (packageJson.dependencies['grunt-githooks']) {
        delete packageJson.dependencies['grunt-githooks'];
    }

    // Add postinstall npm script to set up githooks to package.json
    if (packageJson.scripts) {
        packageJson.scripts['postinstall'] = 'grunt setup-githooks';
    } else {
        packageJson.scripts = {
            'postinstall': 'grunt setup-githooks'
        };
    }

    // Write package.json file back out
    this.writeFileFromString(JSON.stringify(packageJson, null, 2), packageJsonPath); // Our package.json uses 2 spaces as whitespace

    // Copy files
    this.copy('tasks.js', 'tasks/hooks/tasks.js');
    this.copy('githooks.js', 'tasks/config/githooks.js');
    this.directory('githooks', 'githooks');
    this.directory('githooks-env', 'tasks/githooks-env');

    //  Update Gruntfile
    var gruntfilePath = './Gruntfile.js';

    var self = this;
    // check gruntfile exists
    fs.exists(gruntfilePath, function(exists) {
        if (exists) {
            console.log('Patching Gruntfile.js...');
            var gruntfileData = self.readFileAsString(gruntfilePath);
            var lintAllTask = 'grunt.registerTask(\'lint-all\'';

            // Build up which linters to load the lint-all task with
            var activeLinters = '\'lint:prod\'';
            if (self.includeSass) {
                activeLinters += ', \'scsslint\'';
            }
            // Create the new lint-all task
            var newLintTask = 'grunt.registerTask(\'lint-all\', \'Task for running all linters in the project\', [' + activeLinters + ']);';
            var lastGruntTask;
            var lastGruntTaskReplacement;

            if (gruntfileData.indexOf(lintAllTask) === -1) {
                // Build a new last grunt task for insertion
                var lastGruntTaskSliced = gruntfileData.slice(gruntfileData.lastIndexOf('grunt.registerTask('));
                lastGruntTask = lastGruntTaskSliced.slice(0, lastGruntTaskSliced.indexOf(';') + 1);
                lastGruntTaskReplacement = lastGruntTask + '\n    ' + newLintTask;
            } else {
                // Replace existing lint-all task
                var lintAllSlice = gruntfileData.slice(gruntfileData.lastIndexOf(lintAllTask));
                lastGruntTask = lintAllSlice.slice(0, lintAllSlice.indexOf(';') + 1);
                lastGruntTaskReplacement = newLintTask;
            }
            // Insert the new last task, with lint-all task included
            gruntfileData = gruntfileData.replace(lastGruntTask, lastGruntTaskReplacement);

            // Write out the new Gruntfile
            self.writeFileFromString(gruntfileData, gruntfilePath);
        }
    });

    //  Update .gitignore
    var gitIgnorePath = './.gitignore';

    // check gitignore exists
    fs.exists(gitIgnorePath, function(exists) {
        if (exists) {
            var gitIgnoreData = self.readFileAsString(gitIgnorePath);
            var gitHooksIgnore = 'githooks/pre-push.sh';
            if (gitIgnoreData.indexOf(gitHooksIgnore) === -1) {
                console.log('Patching .gitignore...');
                gitIgnoreData = gitIgnoreData + '\n' + gitHooksIgnore;

                self.writeFileFromString(gitIgnoreData, gitIgnorePath);
            }
        }
    });

    var self = this;

    // Leverage npm scripts to set up githooks
    this.on('end', function() {
        var child = this.spawnCommand('npm', ['install']);

        child.on('exit', function() {
            self.emit('githooks:done');

            console.log('Githooks installed!');
        });
    });
};

module.exports = GithooksGenerator;
