'use strict';
var _ = require('lodash');
var fs = require('fs');
var util = require('util');
var yeoman = require('yeoman-generator');
var chalk = require('chalk');

var _confirmLivingStyleGuide = false;

var LivingStyleGuideGenerator = module.exports = function LivingStyleGuideGenerator(args, options, config) {
    yeoman.generators.Base.apply(this, arguments);

    if (this.options.projectSlug) {
        this.projectSlug = this.options.projectSlug;
    } else {
        // While patching, we won't have a slug
        this.projectSlug = 'Living Styleguide';
    }
    _confirmLivingStyleGuide = this.options.confirmLivingStyleGuide;

    var self = this;

    this.on('end', function() {
        if (!_confirmLivingStyleGuide) {
            console.log(chalk.bold.yellow('\nUpdating dependencies...'));

            // Wait for npm install to finish, then event up that we're done
            self.spawnCommand('npm', ['install']).on('exit', function() {
                console.log(chalk.bold.yellow('\nLiving Style Guide is done!'));
                self.emit('livingStyleGuide:done');
            });
        }
    });
};

util.inherits(LivingStyleGuideGenerator, yeoman.generators.Base);

LivingStyleGuideGenerator.prototype.askFor = function askFor() {
    var callback = this.async();

    var prompts = [
        {
            when: function(response) {
                return !_confirmLivingStyleGuide;
            },
            type: 'confirm',
            name: 'confirmLivingStyleGuide',
            message: 'WARNING: The Living Styleguide is only compatible with projects using grunt-sass (libsass) and node-sass-import-once. Do you wish to proceed?',
            default: false
        }
    ];

    this.prompt(prompts, function(props) {
        // Apply prompts
        this.confirmLivingStyleGuide = _confirmLivingStyleGuide || props.confirmLivingStyleGuide;

        // Exit if not confirmed
        if (!this.confirmLivingStyleGuide) {
            process.exit(0);
        }

        callback();
    }.bind(this));
};

LivingStyleGuideGenerator.prototype.app = function app() {
    // Ability to force file overwrites for testing
    if (this.options.forceOverwrites) {
        this.conflicter.force = true;
    }

    // Add all of the Living Style Guide specific tasks
    this.copy('tasks/config/connect.js', 'tasks/config/connect.js');
    this.copy('tasks/config/requirejs.js', 'tasks/config/requirejs.js');
    this.copy('tasks/config/watch.js', 'tasks/config/watch.js');
    this.copy('tasks/config/sass.js', 'tasks/config/sass.js');
    this.directory('tasks/styleguide', 'tasks/styleguide');

    // For patching Living Style Guide to existing Adaptive projects:
    // -----
    //  Update Gruntfile
    var gruntfilePath = './Gruntfile.js';

    var self = this;
    // check gruntfile exists
    fs.exists(gruntfilePath, function(exists) {
        if (exists) {
            console.log('Patching Gruntfile.js...');
            var gruntfileData = self.readFileAsString(gruntfilePath);
            var previewTaskStart = 'grunt.registerTask(\'preview\',';

            if (gruntfileData.lastIndexOf(previewTaskStart) !== -1) {
                // Find the preview task for re-writing
                var slicedPreviewTaskString = gruntfileData.slice(gruntfileData.lastIndexOf(previewTaskStart));
                var previewTask = slicedPreviewTaskString.slice(0, slicedPreviewTaskString.indexOf(';'));

                if (previewTask.indexOf('build_dev') !== -1) {
                    var newPreviewTask = previewTask.replace('build_dev', 'build-with-styleguide');

                    // Re-write the preview task
                    gruntfileData = gruntfileData.replace(previewTask, newPreviewTask);
                }

                // Write out the new Gruntfile
                self.writeFileFromString(gruntfileData, gruntfilePath);
            }
        }
    });

    // Read styleguide dependencies from generator's npmDependencies file
    var generatorDependencies = JSON.parse(this.readFileAsString(this.sourceRoot() + '../../../app/dependencies/npmDependencies.json'));
    var _livingStyleGuideNpmDevDependencies = generatorDependencies.livingStyleGuide;

    // Add npm dev dependencies
    console.log('Adding The Living Styleguide\'s dev dependencies...');

    for (var npmDep in _livingStyleGuideNpmDevDependencies) {
        if (_livingStyleGuideNpmDevDependencies.hasOwnProperty(npmDep)) {
            self.npmInstall(npmDep + '@' + _livingStyleGuideNpmDevDependencies[npmDep], {
                'saveDev': true
            });
        }
    }
};

LivingStyleGuideGenerator.prototype.files = function files() {
    this.template('styleguide.dust', 'styleguide/styleguide.dust');
    this.template('styleguide.scss', 'styleguide/styleguide.scss');
    this.template('styleguide-components.dust', 'styleguide/styleguide-components.dust');
    this.template('config.js', 'styleguide/config.js');
    this.template('context.js', 'styleguide/context.js');
    this.template('index.html', 'styleguide/index.html');
    this.template('README.md', 'styleguide/README.md');
    this.template('ui.js', 'styleguide/ui.js');
    this.template('_styleguide-example-component.dust', 'app/components/styleguide-example/styleguide-example.dust');
    this.directory('data', 'styleguide/data');
    this.template('table-of-contents.dust', 'styleguide/table-of-contents.dust');
};
